from ngsolve import *
from netgen.geom2d import unit_square

mesh = Mesh(unit_square.GenerateMesh(maxh=0.3))


fes_ho = HCurl(mesh, order=3)
fes_lo = HCurl(mesh, order=0)

gf_ho = GridFunction(fes_ho)
gf_lo = GridFunction(fes_lo)

#Set something
gf_ho.Set( CoefficientFunction( (x**2*y*(1-y),-y**3*x**2) ) )
Draw(gf_ho, mesh, "gf_ho")


#Interpolate high order solution to low order space
gf_lo.Set(gf_ho)
Draw(gf_lo, mesh, "gf_lo")


#Now the Projector
gf_proj = GridFunction(fes_ho)

bt = BitArray(fes_ho.FreeDofs())
bt[:]=True
bt[mesh.nedge:] = False
P = Projector(bt, True)
gf_proj.vec.data = P*gf_ho.vec
Draw(gf_proj, mesh, "gf_proj")



#Difference
print("diff proj - lo=", sqrt(Integrate( (gf_proj-gf_lo)*(gf_proj-gf_lo), mesh) ))
print("err proj =", sqrt(Integrate( (gf_proj-gf_ho)*(gf_proj-gf_ho), mesh) ))
print("err lo   =", sqrt(Integrate( (gf_lo-gf_ho)*(gf_lo-gf_ho), mesh) ))
